/**********************************************************************/
/** This file contains the code to compute LIV estimates             **/
/**    and run the estimation procedure over the Monte-Carlo         **/
/**    datasets given the master data, selection mechanism,          **/
/**    information about policy regime (education subsidy available  **/
/**    or not) and the set of selected instruments. Uses local       **/
/**    quadratic regression to estimate the slope of the function    **/
/**    E(y|X). Routines in here are called by the main program,      **/
/**    LIV.do.                                                       **/
/**                                                                  **/
/** Monica Costa Dias                                                **/
/** September 2008                                                   **/
/**********************************************************************/



*********************************************************************
** LIV estimation procedure: estimate ATT as the integral over the **
**   distribution of x|d=1 of ATT(x)                               **
**                                                                 **
**   need to know: observed outcome (y)                            **
**   input as global: education status ($d)                        **
**                    instruments ($IVvar)                         **
*********************************************************************

capture program drop LIV
program define LIV
		
	qui cap drop p                               /* ATT in region 0         */
	qui probit $d $IVvar if x==0
	qui predict p if x==0
	qui LIV_ATTx
	scalar LIVx0 = LIVx

	qui drop p                                   /* ATT in region 1         */
	qui probit $d $IVvar if x==1
	qui predict p if x==1
	qui LIV_ATTx
	scalar LIVx1 = LIVx

	qui count if $d==1 & x==0                    /* distribution of x       */
	scalar N0=r(N)
	qui count if $d==1 & x==1
	scalar N1=r(N)

	global LIV  = (LIVx1*N1 + LIVx0*N0)/(N1+N0)  /* LIV estimate of the ATT */

end



*********************************************************************
** LIV estimation procedure: estimate ATT conditional on x, ATT(x) **
**                                                                 **
**   need to know: propensity score (p)                            **
**                 observed outcome (y)                            **
**   input as global: bandwidth ($h)                               **
**                    step to form grid in p ($step)               **
*********************************************************************

capture program drop LIV_ATTx
program define LIV_ATTx

	qui gen cp  = 0                                        /* generate variable to contain centred propensity score for LQR */
	qui gen cp2 = 0                                        /*   and its square                                              */
	qui gen k   = 0                                        /* generate variable to contain kernel weights                   */


	** Establish interval of integration to estimate the ATT:  ** 
	**    between min and max observable p                     **
	*************************************************************
	qui sum p
	local mini = int(r(min)/$step+2)                             /* `mini'*$step is the minimum value of observable p       */
	local maxi = int(r(max)/$step-1)                             /* `maxi'*$step is the maximum value of observable p       */


	** Estimating ATT for individuals drawing unobservable niu **
	**    below p over the whole observable domain of p        **
	*************************************************************
	scalar LIVacc = 0
	forvalues i = `mini'/`maxi' {

		scalar pointp = `i'*$step

		qui replace cp  = p-pointp                            /* centered regressor p (when drawing niu=pointp=`i'*step) */
		qui replace cp2 = cp^2
		qui replace k   = (abs(cp/$h)<=1)*(1-(cp/$h)^2)*3/4   /* Epanechnikov weights for local quadratic regression     */

		** estimating MTE(niu=pointp) using LIV **
		******************************************
		qui reg y cp cp2 [iw=k]                               /* local quadratic regression of eanings on p              */
		matrix e = e(b)
		scalar LIVp = e[1,1]                                  /* LIV estimate of MTE(niu=pointp)                         */

		** estimating ATT for individuals drawing niu<=pointp **
		********************************************************
		scalar LIVacc = LIVacc+LIVp
		scalar LIVx`i' = LIVacc/(`i'-`mini'+1)

		** kernel estimate of the pdf of p among treated at p=pointp **
		***************************************************************
		qui sum k if $d==1
		scalar pdf`i' = r(mean)/$h

	}


	** Integrating ATT(p) over the whole observable domain     **
	**    of p to obtain ATT                                   **
	*************************************************************
	scalar cdf  = 0
	scalar LIVx = 0
	forvalues i=`mini'/`maxi' {
		scalar cdf  = cdf + pdf`i'
		scalar LIVx = LIVx + (LIVx`i'*pdf`i')
	}
	scalar LIVx = LIVx/cdf


	** cleaning the data **
	***********************
	qui drop cp cp2 k 

end



**************************************************************
** Monte Carlo replications of the LIV estimation procedure **
**************************************************************

cap program drop MCLIV
program define MCLIV


	** Monte-Carlo replications **
	******************************
	forvalues i=1/200 {

		display "MC replication ", `i'

		local lb = (`i'-1)*$N+1                     /* read data              */
		local ub = `i'*$N
		qui use $dtafile in `lb'/`ub', clear

		qui gen y  = $d*ln(y1)+(1-$d)*ln(y0)        /* observed outcome       */
		qui LIV
		global LIV`i' = $LIV

	}


	** Storing estimates **
	***********************
	drop _all
	set obs 200
	gen trial = _n
	gen LIV   = .
	forvalues i=1/200 {
		qui replace LIV = ${LIV`i'} if trial == `i'
	}

end




